from collections.abc import Callable
import logging

import hmac
import hashlib

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand, CommandParser
from django.conf import settings
from django_coturn.services import (
    delete_turn_secrets,
    create_turn_secret,
    get_or_update_turn_admin,
    get_or_update_turn_user,
)

logger = logging.getLogger(__name__)

User = get_user_model()


class Command(BaseCommand):
    def add_arguments(self, parser: CommandParser) -> None:
        subparsers = parser.add_subparsers(
            title="subcommands", dest="subcommand", required=True
        )

        secret_cmd_parser = subparsers.add_parser("turn_secret")
        secret_cmd_parser.set_defaults(subcommand=self.handle_turn_secret)

        turn_admin_parser = subparsers.add_parser("turn_admin")
        turn_admin_parser.set_defaults(subcommand=self.handle_turn_admin)

        turn_user_parser = subparsers.add_parser("turn_user")
        turn_user_parser.set_defaults(subcommand=self.handle_turn_user)

    def handle_turn_user(self, *args, **kwargs):
        for user in User.objects.all():
            turn_user = get_or_update_turn_user(user)
            self.stdout.write(f"Created {turn_user} from {user}")

    def handle_turn_secret(self, *args, **kwargs):
        delete_turn_secrets()
        new_secret = create_turn_secret()
        self.stdout.write(f"Created {new_secret} from COTURN_SECRET_KEY")

    def handle_turn_admin(self, *args, **kwargs):
        for user in User.objects.filter(is_superuser=True).all():
            turn_admin = get_or_update_turn_admin(user)
            self.stdout.write(f"Created {turn_admin} from {user}")

    def handle(self, subcommand: Callable, *args, **kwargs):
        return subcommand(*args, **kwargs)



from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django.views.generic import View

from .services import create_turn_api_credentials


@method_decorator(login_required, name="dispatch")
class CreateCoturnCredential(View):
    """
    Generate a token conforming to TURN REST Server API spec outlined in draft-uberti-behave-turn-rest-00
    https://www.ietf.org/proceedings/87/slides/slides-87-behave-10.pdf
    """

    def get(self, request):
        username, password = create_turn_api_credentials(request.user.email)



from django import dispatch
from django.contrib.auth import get_user_model
from django.dispatch import receiver
from django.db.models.signals import post_save
from .models import TurnUser, TurnAdmin
from .services import get_or_update_turn_user

User = get_user_model()


@receiver(post_save, sender=User, dispatch_uid="django_coturn_user_post_save")
def create_turn_models(sender, instance, **kwargs) -> TurnUser:
    return get_or_update_turn_user(instance)



"""
dj-stripe Migrations Tests
"""
import string
import random
import pytest
from django.conf import settings
from django.test import TestCase, override_settings
from django.core.exceptions import ImproperlyConfigured

from django_coturn.settings import coturn_settings


class TestCoturnSettings(TestCase):
    def setUp(self):
        return super().setUp()

    @override_settings(COTURN_AUTH_STRATEGY="invalid")
    def test_invalid_auth_strategy(self):
        with pytest.raises(ImproperlyConfigured):
            coturn_settings.COTURN_AUTH_STRATEGY

    @override_settings(COTURN_REALM=None, COTURN_SECRET_KEY=None)
    def test_required_settings(self):
        with pytest.raises(ImproperlyConfigured):
            coturn_settings.COTURN_REALM
        with pytest.raises(ImproperlyConfigured):
            coturn_settings.COTURN_SECRET_KEY

    @override_settings(
        COTURN_SECRET_KEY="".join(
            random.choices(string.ascii_uppercase + string.digits, k=128)
        )
    )
    def test_invalid_secret_key(self):
        with pytest.raises(ImproperlyConfigured):
            coturn_settings.COTURN_SECRET_KEY



import pytest
from django.conf import settings
from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings

from django_coturn.enum import CoturnAuthStrategy
from django_coturn.models import TurnAdmin, TurnUser
from django_coturn.settings import coturn_settings


ADMIN_EMAIL = "admin@test.com"
USER_EMAIL = "user@test.com"


class TestUserTurnAdminFK(TestCase):
    @override_settings(
        COTURN_USER_MODEL="testapp.CustomUser",
        COTURN_USER_MODEL_REQUEST_CALLBACK=(lambda request: request.user),
    )
    def setUp(self):
        return super().setUp()

    @override_settings(COTURN_AUTH_STRATEGY=CoturnAuthStrategy.TURN_REST_API)
    def test_admin_user_created_for_django_superuser(self):
        UserModel = get_user_model()
        password = UserModel.objects.make_random_password()
        admin_user = UserModel(
            email=ADMIN_EMAIL, username=ADMIN_EMAIL, is_superuser=True
        )
        admin_user.set_password(password)
        admin_user.save()



import base64
import hashlib
import hmac

import logging
from typing import Tuple
from datetime import timedelta, datetime, timezone
from django.conf import settings
from django.contrib.auth import get_user_model
from django.shortcuts import get_object_or_404

from .models import TurnAdmin, TurnSecret, TurnUser
from .settings import coturn_settings

# TURN REST API authentication strategy
# https://www.ietf.org/proceedings/87/slides/slides-87-behave-10.pdf

logger = logging.getLogger(__name__)
User = get_user_model()


def _get_expiration_timestamp() -> int:
    now = datetime.utcnow()
    expiration_ts = (
        now + timedelta(seconds=coturn_settings.COTURN_CREDENTIAL_MAX_AGE)
    ).timestamp()
    return int(expiration_ts)


def _create_turn_api_username(email: str) -> str:
    expiration_ts = _get_expiration_timestamp()
    username = "{}:{}".format(email, expiration_ts)
    return username


def _create_turn_api_password(username: str) -> str:
    secret = get_object_or_404(TurnSecret, realm=coturn_settings.COTURN_REALM).first()
    token = hmac.new(
        secret.value.encode("utf-8"), username.encode("utf-8"), hashlib.sha1
    )
    encoded_token = base64.b64encode(token.digest()).decode("utf-8")
    return encoded_token


def create_turn_api_credentials(username: str) -> Tuple[str, str]:
    username = _create_turn_api_username(username)
    password = _create_turn_api_password(username)
    return username, password


def create_turn_admin(user: User, default_password="CHANGEME") -> TurnAdmin:
    return TurnAdmin.objects.create(
        name=user.email,
        django_user_id=user.id,
        password=default_password,
    )


def delete_turn_secrets():
    for s in TurnSecret.objects.using("coturn").all():
        s.delete(using="coturn")


def create_turn_secret() -> TurnSecret:
    realm = settings.COTURN_REALM
    secret = settings.COTURN_SECRET_KEY
    new_secret = TurnSecret(realm=realm, value=secret)
    new_secret.save(using="coturn")
    return new_secret


def get_or_update_turn_user(user: User) -> TurnUser:
    try:
        turn_user = TurnUser.objects.get(django_user_id=user.id)
        logger.debug(
            f"Found existing TurnUser id={turn_user.id} django_user_id={turn_user.django_user_id}"
        )
    except TurnUser.DoesNotExist:
        password = User.objects.make_random_password()
        hmackey = hmac.new(
            settings.COTURN_SECRET_KEY.encode("utf-8"),
            password.encode("utf-8"),
            hashlib.sha1,
        )
        hmackey.update(settings.COTURN_REALM.encode("utf-8"))
        turn_user = TurnUser.objects.create(
            name=user.email,
            realm=settings.COTURN_REALM,
            django_user_id=user.id,
            hmackey=hmackey.hexdigest(),
        )
        logger.debug(
            f"Created new TurnUser id={turn_user.id} django_user_id={turn_user.django_user_id}"
        )
    return turn_user


def get_or_update_turn_admin(user: User) -> TurnAdmin:
    try:
        turn_admin = TurnAdmin.objects.get(name=user.email)
        if turn_admin.django_user_id is None:
            turn_admin.django_user_id = user.id
            turn_admin.save()
            logger.info(
                f"Linked TurnAdmin id={turn_admin.id} to django_user_id={turn_admin.django_user_id}"
            )
    except TurnAdmin.DoesNotExist:
        turn_admin = TurnAdmin.objects.create(
            name=user.email,
            django_user_id=user.id,
            password=User.objects.make_random_password(),
        )
    return turn_admin



from unittest.mock import patch
from django.test import TestCase, override_settings
from datetime import datetime, timezone, tzinfo
from freezegun import freeze_time

from django_coturn.services import (
    create_turn_api_credentials,
    _get_expiration_timestamp,
)


class TestServices(TestCase):
    @override_settings(COTURN_CREDENTIAL_MAX_AGE=3600)
    def test_get_expiration_timestamp_default(self):
        expected = int(datetime(2021, 9, 14, 1, tzinfo=timezone.utc).timestamp())
        with freeze_time("2021-09-14 00:00:00", tz_offset=0):
            actual = _get_expiration_timestamp()
        assert expected == actual

    # def test_create_turn_api_credentials(self):
    #     mock_datetime_now = datetime
    #     email = "test-user@test.com"
    #     username, password = create_turn_api_credentials(email)



# References: https://github.com/coturn/coturn/blob/master/turndb/schema.sql

from django.db import models

from django.contrib.auth.models import AbstractUser

from .settings import coturn_settings


class TurnAdmin(models.Model):
    """
    A coturn admin_user row is created automatically for superusers
    """

    name = models.CharField(unique=True, max_length=32)
    realm = models.CharField(max_length=127, blank=True, null=True)
    password = models.CharField(max_length=127)
    django_user_id = models.BigIntegerField(null=True)

    class Meta:
        db_table = "admin_user"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "name"], name="unique_admin_username_per_realm"
            ),
        )


class AllowedPeerIp(models.Model):
    realm = models.CharField(max_length=127, blank=True, null=True)
    ip_range = models.CharField(max_length=256, blank=True, null=True)

    class Meta:
        db_table = "allowed_peer_ip"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "ip_range"], name="unique_allowed_ip_range_realm"
            ),
        )


class DeniedPeerIp(models.Model):
    realm = models.CharField(max_length=127, blank=True, null=True)
    ip_range = models.CharField(max_length=256, blank=True, null=True)

    class Meta:
        db_table = "denied_peer_ip"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "ip_range"], name="unique_denied_ip_range_realm"
            ),
        )


class OauthKey(models.Model):
    kid = models.CharField(unique=True, max_length=128)
    ikm_key = models.CharField(max_length=256)
    timestamp = models.BigIntegerField()
    lifetime = models.IntegerField()
    as_rs_alg = models.CharField(max_length=64)
    realm = models.CharField(max_length=127)

    class Meta:
        db_table = "oauth_key"


class TurnOriginToRealm(models.Model):
    origin = models.CharField(unique=True, max_length=127)
    realm = models.CharField(max_length=127)

    class Meta:
        db_table = "turn_origin_to_realm"


class TurnRealmOption(models.Model):
    realm = models.CharField(max_length=127, blank=True, null=True)
    opt = models.CharField(max_length=32, blank=True, null=True)
    value = models.CharField(max_length=128, blank=True, null=True)

    class Meta:
        db_table = "turn_realm_option"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "opt"], name="unique_opt_per_realm"
            ),
        )


class TurnSecret(models.Model):
    realm = models.CharField(max_length=127)
    value = models.CharField(max_length=127)

    class Meta:
        db_table = "turn_secret"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "value"], name="unique_value_per_realm"
            ),
        )


class TurnUser(models.Model):
    """
    This data model is only used by Coturn's long-term credential strategy
    """

    realm = models.CharField(max_length=127, blank=True, default="")
    name = models.CharField(max_length=512, blank=True, null=True)
    hmackey = models.CharField(max_length=128, blank=True, null=True)
    django_user_id = models.BigIntegerField()

    class Meta:
        db_table = "turnusers_lt"
        constraints = (
            models.UniqueConstraint(
                fields=["realm", "name"], name="unique_username_per_realm"
            ),
            models.UniqueConstraint(
                fields=["realm", "django_user_id"], name="unique_django_user_per_realm"
            ),
        )



import os
import environ

env = environ.Env()

DEBUG = True
TIME_ZONE = "UTC"
USE_TZ = True
PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR = os.path.dirname(PROJECT_DIR)

db_config = env.db("DATABASE_URL")
db_config["ENGINE"] = "django.db.backends.postgresql"
coturn_db_config = env.db("COTURN_DATABASE_URL")
coturn_db_config["ENGINE"] = "django.db.backends.postgresql"
DATABASES = {"default": db_config, "coturn": coturn_db_config}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.request",
            ]
        },
    }
]

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    "tests",
    "tests.apps.testapp",
    "django_coturn",
]

MIDDLEWARE = (
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

STATIC_URL = "/static/"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

SECRET_KEY = "unGPquS03pKnzfQblEpV9KQtFRwgkGaNyS5Ijra7JkM56P9xbE"

AUTH_USER_MODEL = "testapp.CustomUser"
COTURN_REALM = "north.gov"
COTURN_SECRET_KEY = SECRET_KEY



from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Model

from .enum import CoturnAuthStrategy


class CoturnSettings:
    @property
    def COTURN_AUTH_STRATEGY(self) -> CoturnAuthStrategy:
        auth_strategy = getattr(
            settings, "COTURN_AUTH_STRATEGY", CoturnAuthStrategy.TURN_REST_API
        )
        if not isinstance(auth_strategy, CoturnAuthStrategy):
            raise ImproperlyConfigured(
                "COTURN_AUTH_STRATEGY must be set to \
                coturn.enum.CoturnAuthStrategy.TURN_REST_API (default) or \
                or coturn.enum.CoturnAuthStrategy.LONG_TERM_CREDENTIALS"
            )
        return auth_strategy

    @property
    def COTURN_CREDENTIAL_MAX_AGE(self) -> int:
        return int(getattr(settings, "COTURN_CREDENTIAL_MAX_AGE", 60))

    @property
    def COTURN_REALM(self) -> str:
        coturn_realm = getattr(settings, "COTURN_REALM", None)
        if not coturn_realm:
            raise ImproperlyConfigured("COTURN_REALM must be set in settings.py")
        return coturn_realm

    @property
    def COTURN_SECRET_KEY(self) -> str:
        secret = getattr(settings, "COTURN_SECRET_KEY", None)
        if not secret:
            raise ImproperlyConfigured("COTURN_SECRET_KEY must be set in settings.py")
        if len(secret) > 127:
            raise ImproperlyConfigured(
                "COTURN_SECRET_KEY must be less than 127 characters in length"
            )
        return secret

    def get_user_model_string(self) -> str:
        """Get the configured subscriber model as a module path string."""
        return getattr(settings, "COTURN_USER_MODEL", settings.AUTH_USER_MODEL)

    def get_user_model(self) -> Model:
        """
        Attempt to read settings.COTURN_USER_MODEL.
        This methods falls back to AUTH_USER_MODEL if COTURN_USER_MODEL is not set.
        Returns the subscriber model that is active in this project.
        """
        model_name = self.get_user_model_string()

        # Attempt a Django 1.7 app lookup
        try:
            user_model = django_apps.get_model(model_name)
        except ValueError:
            raise ImproperlyConfigured(
                "COTURN_USER_MODEL must be of the form 'app_label.model_name'."
            )
        except LookupError:
            raise ImproperlyConfigured(
                "COTURN_USER_MODEL refers to model '{model}' "
                "that has not been installed.".format(model=model_name)
            )

        return user_model


coturn_settings = CoturnSettings()



# Generated by Django 3.2.7 on 2021-09-19 20:43

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="AllowedPeerIp",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("realm", models.CharField(blank=True, max_length=127, null=True)),
                ("ip_range", models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                "db_table": "allowed_peer_ip",
            },
        ),
        migrations.CreateModel(
            name="DeniedPeerIp",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("realm", models.CharField(blank=True, max_length=127, null=True)),
                ("ip_range", models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                "db_table": "denied_peer_ip",
            },
        ),
        migrations.CreateModel(
            name="OauthKey",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("kid", models.CharField(max_length=128, unique=True)),
                ("ikm_key", models.CharField(max_length=256)),
                ("timestamp", models.BigIntegerField()),
                ("lifetime", models.IntegerField()),
                ("as_rs_alg", models.CharField(max_length=64)),
                ("realm", models.CharField(max_length=127)),
            ],
            options={
                "db_table": "oauth_key",
            },
        ),
        migrations.CreateModel(
            name="TurnAdmin",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=32, unique=True)),
                ("realm", models.CharField(blank=True, max_length=127, null=True)),
                ("password", models.CharField(max_length=127)),
                ("django_user_id", models.BigIntegerField(null=True)),
            ],
            options={
                "db_table": "admin_user",
            },
        ),
        migrations.CreateModel(
            name="TurnOriginToRealm",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("origin", models.CharField(max_length=127, unique=True)),
                ("realm", models.CharField(max_length=127)),
            ],
            options={
                "db_table": "turn_origin_to_realm",
            },
        ),
        migrations.CreateModel(
            name="TurnRealmOption",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("realm", models.CharField(blank=True, max_length=127, null=True)),
                ("opt", models.CharField(blank=True, max_length=32, null=True)),
                ("value", models.CharField(blank=True, max_length=128, null=True)),
            ],
            options={
                "db_table": "turn_realm_option",
            },
        ),
        migrations.CreateModel(
            name="TurnSecret",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("realm", models.CharField(max_length=127)),
                ("value", models.CharField(max_length=127)),
            ],
            options={
                "db_table": "turn_secret",
            },
        ),
        migrations.CreateModel(
            name="TurnUser",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("realm", models.CharField(blank=True, default="", max_length=127)),
                ("name", models.CharField(blank=True, max_length=512, null=True)),
                ("hmackey", models.CharField(blank=True, max_length=128, null=True)),
                ("django_user_id", models.BigIntegerField()),
            ],
            options={
                "db_table": "turnusers_lt",
            },
        ),
        migrations.AddConstraint(
            model_name="turnuser",
            constraint=models.UniqueConstraint(
                fields=("realm", "name"), name="unique_username_per_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="turnuser",
            constraint=models.UniqueConstraint(
                fields=("realm", "django_user_id"), name="unique_django_user_per_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="turnsecret",
            constraint=models.UniqueConstraint(
                fields=("realm", "value"), name="unique_value_per_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="turnrealmoption",
            constraint=models.UniqueConstraint(
                fields=("realm", "opt"), name="unique_opt_per_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="turnadmin",
            constraint=models.UniqueConstraint(
                fields=("realm", "name"), name="unique_admin_username_per_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="deniedpeerip",
            constraint=models.UniqueConstraint(
                fields=("realm", "ip_range"), name="unique_denied_ip_range_realm"
            ),
        ),
        migrations.AddConstraint(
            model_name="allowedpeerip",
            constraint=models.UniqueConstraint(
                fields=("realm", "ip_range"), name="unique_allowed_ip_range_realm"
            ),
        ),
    ]



from io import StringIO

from django.contrib.auth import get_user_model
from django.core.management import call_command
from django.test import TestCase

User = get_user_model()


class TestCommand(TestCase):

    databases = ("coturn", "default")

    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create(
            email="admin@test.com", password="testing1234", is_superuser=True
        )

    def test_secret_sync(self):
        out = StringIO()
        call_command("coturn_sync", "turn_secret", stdout=out)
        assert "Created TurnSecret" in out.getvalue()

    def test_admin_sync(self):
        out = StringIO()
        call_command("coturn_sync", "turn_admin", stdout=out)
        assert f"Created TurnAdmin object ({self.user.id})" in out.getvalue()

    def test_user_sync(self):
        out = StringIO()
        call_command("coturn_sync", "turn_user", stdout=out)
        assert f"Created TurnUser object ({self.user.id})" in out.getvalue()



from django.apps import AppConfig


class DjangoCoturnConfig(AppConfig):
    name = "django_coturn"

    def ready(self):
        try:
            import django_coturn.signals  # noqa F401
        except ImportError:
            pass



from enum import Enum


class CoturnAuthStrategy(Enum):
    LONG_TERM_CREDENTIALS = "Long Term Credentials"
    TURN_REST_API = "TURN REST API described in https://www.ietf.org/proceedings/87/slides/slides-87-behave-10.pdf"



#!/usr/bin/env python
import os
from typing import List

from setuptools import setup, find_packages
from django_coturn import __version__

long_description: str = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
install_requires = [
    'django>=3.2',
    'psycopg2'
]
python_requires = '>3.6.9'
setup(
    name = 'django_coturn',
    version = __version__,
    packages = find_packages(),
    author = 'Leigh Johnson',
    author_email = 'leigh@bitsy.ai',
    description = 'Django Coturn is a Django app to synchronize django admins/users with Coturn\'s user database. \
        Coturn is an open-source STUN/TURN/ICE server.',
    long_description = long_description,
    license = 'GNU AGPLv3',
    keywords = 'django stun turn ice coturn webrtc',
    url = 'http://github.com/bitsy-ai/django-coturn',
    classifiers = [
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Framework :: Django",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2"
    ],
    zip_safe = False,
    install_requires = install_requires,
    test_suite = 'pytest',
    python_requires=python_requires,
    include_package_data=True
)